/** \file
 */

/*
 *  
 * Title:   Buffer Handling
 * Author(s): Domnic Savio
 *
 * ------------------------------------------------------------------
 * @version: $Id: buffer.h,v 1.5.2.3 2012/07/14 18:12:40 benya Exp $
 * ------------------------------------------------------------------ 
 *
 */

 
#include "buffer.h" 



/** Initializes a buffer. 

    Length will be set to zero and the data array will be filled with 0xBB. (This makes it easier to search for initialized buffers while debugging.) 
   \return 

     - \c false, if b is not a valid pointer

     - \c true, if everything worked fine

*/

bool initBuffer(Buffer *b) 

{

  int i;

  if (b != NULL)

    {

      b->len = 0;

      for (i=0; i<MAX_BUF_LEN; i++) 

	{

	  b->data[i] = (char) 0xBB;

	}

      return true;

    }

  else

    { //error("initBuffer: b points to NULL");

      return false;

    }

}





/** Copy buffer 'src' to buffer 'dst'.

    But not more than MAX_BUF_LEN chars. 



    \param dst [out] will be a copy of src or, in case of failure, an initialized buffer

    \param src [in] to be copied



    \pre

      - src != NULL

      - dst != NULL



    \return

      - \c false, on error 

      - \c true, otherwise



 dst will look like an initialized buffer. Otherwise returns true.

*/

bool copyBuffer(Buffer *dst, Buffer *src)

{

  int i;

  if (dst != NULL && src != NULL)

    {

      if (src->len > MAX_BUF_LEN || src->len < 0)

	{

	  //error("copyBuffer: src-buffer is corrupt");

	  initBuffer(dst); 

	  return false;

	}

      else

	{

	  for (i=0; i<src->len; i++)

	    {

	      dst->data[i] = src->data[i];

	    }

	  dst->len = src->len;

	  return true;

	}

    }

  else

    {

      //error("copyBuffer: dst or src point to NULL");

      return false;

    }

}









/** Fill buffer with one character.



   Fills buffer 'b' with c for the whole length (MAX_BUF_LEN).

   

*/

bool fillBuffer(Buffer *b, char c)

{

    int i;

    if (b != NULL) 

      {    

	for (i=0; i<MAX_BUF_LEN; i++) 

	  {

	    b->data[i] = c;

	  }

	b->len = MAX_BUF_LEN;

	return true;

      }

    else

      { //error("fillBuffer: b points to NULL");

	return false;

      }

}









/** Create buffer from a string.



   Fills buffer 'b' with the contents of str until a '\\0' is encountered or MAX_BUF_LEN is reached. 

In the latter case, bufferFromString will return false, otherwise true. The '\\0' will not be copied. 

  \note 

  This means you cannot use this function to put a NUL-char ('\\0') into the buffer.

*/

bool bufferFromString(Buffer *b, char *str)

{

    int i;

	

    if (b != NULL && str != NULL)

    {

	i = 0;

	while (str[i] != '\0' && i < MAX_BUF_LEN)

	{

		b->data[i] = str[i];

		i++;

	}

	b->len = i;

        return (i <= MAX_BUF_LEN && str[i] == '\0');

    }

    else

      { //error("bufferFromString: b or str points to NULL");

        return false;

      }

}









/** Append a character to the buffer.

   Append character \c c to buffer \c b if length is smaller than MAX_BUF_LEN. If not, the  function will return false, otherwise true.

*/

bool appendBuffer(Buffer *b, char c)

{

  if (b != NULL)

    {

	if (b->len < MAX_BUF_LEN)

	{

	  b->data[b->len] = c;

	  b->len++;

	  return true;

	}

	else

	{

	  //error("tried to append char to completely filled buffer");

	  return false;

	}

    }

  else

    { //error("appendBuffer: b points to NULL");

      return false;

    }



} 
